-- Importált osztályok a Renoise API-ból
local renoise = renoise
local vb = renoise.ViewBuilder()

-- Képfelsorolás
local imageList = {
    "analogfilter_128x128.png",
    "buscompressor_128x128.png",
    "cabinetsimulator_128x128.png",
    "convolver_128x128.png",
    "delay_128x128.png",
    "digitalfilter_128x128.png",
    "distortion_128x128.png",
    "eq10_128x128.png",
    "eq5_128x128.png",
    "eqMixer_128x128.png",
    "exciter_128x128.png",
    "flanger_128x128.png",
    "gate_128x128.png",
    "hydra_128x128.png",
    "lfo_128x128.png",
    "lofi_128x128.png",
    "maximizer_128x128.png",
    "mpReverb_128x128.png",
    "multibandsend_128x128.png",
    "multitapDelay_128x128.png",
    "phaser_128x128.png",
    "send_128x128.png",
    "sidechain_128x128.png",
    "signalfollower_128x128.png",
    "stereoExpander_128x128.png",
}

-- Tooltip-ök és VST nevek felsorolása (ugyanolyan sorrendben, mint a képfelsorolás)
local tooltipsAndVsts = {
    {tooltip = "Analog Filter", vstName  = "Analog Filter"},
    {tooltip = "Bus Compressor", vstName  = "Bus Compressor"},
    {tooltip = "Cabinet Simulator", vstName  = "cabinet simulator"},
    {tooltip = "Convolver IR", vstName  = "Convolver"},
    {tooltip = "Delay", vstName  = "Delay"},
    {tooltip = "Digital Filter", vstName  = "Digital Filter"},
    {tooltip = "Distortion", vstName  = "Distortion 2"},
    {tooltip = "Eq 10", vstName  = "EQ 10"},
    {tooltip = "Eq 5", vstName  = "EQ 5"},
    {tooltip = "Mixer Eq", vstName  = "Mixer EQ"},
    {tooltip = "Exciter", vstName  = "Exciter"},
    {tooltip = "Flanger", vstName  = "Flanger 2"},
    {tooltip = "Gate", vstName  = "Gate 2"},
    {tooltip = "Hydra", vstName  = "*Hydra"},
    {tooltip = "Lfo", vstName  = "*Lfo"},
    {tooltip = "Lofi Math", vstName  = "LofiMat 2"},
    {tooltip = "Maximizer", vstName  = "Maximizer"},
    {tooltip = "mpReverb", vstName  = "mpReverb 2"},
    {tooltip = "Multiband Send", vstName  = "#Multiband Send"},
    {tooltip = "Multitap Delay", vstName  = "Multitap"},
    {tooltip = "Phaser", vstName  = "Phaser 2"},
    {tooltip = "Send", vstName  = "#Send"},
    {tooltip = "Sidechain", vstName  = "#Sidechain"},
    {tooltip = "Signal Follower", vstName  = "*Signal Follower"},
    {tooltip = "Stereo Expander", vstName  = "Stereo Expander"},
}

-- Define the size of the grid (for example, 5x5)
local numRows = 5
local numCols = 5

-- Define the insertDevice function
local function insertDevice(device_name)
    local sng = renoise.song()
    local is_sampleedit = renoise.app().window.active_middle_frame == renoise.ApplicationWindow.MIDDLE_FRAME_INSTRUMENT_SAMPLE_EFFECTS

    device_name = string.gsub(string.gsub(device_name, "[()]+", ""), " (dep.)", "")

    if sng.selected_device and not is_sampleedit then
        sng.selected_track:insert_device_at('Audio/Effects/Native/'..device_name, sng.selected_device_index + 1)
    elseif not is_sampleedit then
        sng.selected_track:insert_device_at('Audio/Effects/Native/'..device_name, #sng.selected_track.devices + 1)
    elseif sng.selected_sample_device then
        sng.selected_sample_device_chain:insert_device_at('Audio/Effects/Native/'..device_name, sng.selected_sample_device_index + 1)
    else
        sng.selected_sample_device_chain:insert_device_at('Audio/Effects/Native/'..device_name, #sng.selected_sample_device_chain.devices + 1)
    end
end

-- Function to create the grid with images, tooltips, and VST names
local function createGridWithImages()
    local gridContent = {}
    local index = 1 -- Index to keep track of the current image, tooltip, and VST name

    for row = 1, numRows do
        local rowContent = {}
        for col = 1, numCols do
            local imageIndex = index
            local tooltip = tooltipsAndVsts[index].tooltip
            local vstName = tooltipsAndVsts[index].vstName

            local cell = vb:button {
                width = 128,
                height = 128,
                bitmap = imageList[imageIndex], -- Use the current image from the list
                notifier = function()
                    -- Handle button press if needed
                    -- Insert the selected VST to the TrackDSP

                    -- Use the insertDevice function to insert the VST
                    insertDevice(vstName)
                end,
                tooltip = tooltip, -- Set the tooltip
            }
            table.insert(rowContent, cell)

            index = index + 1
            if index > #imageList then
                index = 1
            end
        end
        table.insert(gridContent, vb:horizontal_aligner(rowContent))
    end

    -- Create a container for the grid
    local gridContainer = vb:column(gridContent)

    -- Show the grid container in a dialog
    local dialogTitle = "Visual Toolbar for Native DSP"
    local dialog = renoise.app():show_custom_dialog(dialogTitle, gridContainer)
end

-- Az illesztési logika Native DSP esetén
local function insertNativeDspByName(dspName)
    local selectedTrack = renoise.song().selected_track
    if selectedTrack then
        local dspDevice = selectedTrack:insert_device_at(dspName, selectedTrack:last_effect_number() + 1)
        -- DSP beállítások, ha szükséges
    end
end

-- Az illesztési logika VST esetén
local function insertVSTByName(vstName)
    local selectedTrack = renoise.song().selected_track
    if selectedTrack then
        if renoise.tool():load_plugin(vstName) then
            local effectIndex = selectedTrack:insert_effect_at(vstName, selectedTrack:last_effect_number() + 1)
            -- VST beállítások, ha szükséges
        else
            print("Nem található vagy betölthető a kiválasztott VST vagy DSP.")
        end
    end
end

-- Először hozzuk létre a gombokat a képfelsorolással
-- createGridWithImages()

-- Menu
renoise.tool():add_menu_entry {
  name = "Main Menu:Tools:Hotelsinus:Visual Toolbar for Native DSP",
  invoke = function()
    createGridWithImages()
  end
}

-- Key Binding
renoise.tool():add_keybinding {
  name = "Global:Tools:Visual Toolbar for Native DSP",
  invoke = function()
    createGridWithImages()
  end
}
